/*
  LcdKey.c

  release : 0.0.2 - 19 Feb 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
 */

#include "LcdKey.h"

#define LCD_INIT 1
#define LCD_KEY 2
#define LCD_NOKEY 3
#define LCD_KEYOK 4
#define LCD_REPEAT 5
#define LCD_SELECT 6
#define LCD_EXIT 7

static const uint8_t UNSCALED_LEVELS[7] = {86,60,40,23,7,0};
static int16_t LEVELS[6];
static uint16_t DEBOUNCE;
static uint16_t DELAY;
static uint8_t PIN;
static uint16_t REPEAT;
static uint32_t EXIT;
static uint32_t millis_old;

enum LCDKEY LcdKeyCheck()
{
  int16_t anaIn = analogRead(PIN);
  uint8_t i;
  for (i = 0;i < 6;i++) if (anaIn >= LEVELS[i]) return (enum LCDKEY)i;
  return LCDKEY_NONE;
}

void LcdKeyInit(uint8_t pin,uint16_t debounce,uint16_t delay,uint8_t repeat,uint8_t exit) {
  DEBOUNCE = debounce;
  DELAY = delay;
  PIN = pin;
  REPEAT = 1000/repeat;
  EXIT = (long)exit * 1000;
  int16_t anaIn = analogRead(PIN);
  uint8_t i;
  for (i = 0;i < 6; i++) LEVELS[i] = (long)UNSCALED_LEVELS[i] * anaIn / 100;
}

enum LCDKEY LcdKeyPress() {
  static uint8_t status = LCD_INIT;
  enum LCDKEY key = LcdKeyCheck();
  switch (status) {
    case LCD_INIT:
    case LCD_EXIT:
      if (key != LCDKEY_NONE) {
        millis_old = millis();
        key = LCDKEY_NONE;
        status = LCD_KEY;
      }
      break;
    case LCD_KEY:
      if (key == LCDKEY_NONE) {
        millis_old = millis();
        status = LCD_NOKEY;
      }
      if (((millis() - millis_old) & 0x7FFF) > DEBOUNCE) {
        if (key == LCDKEY_SELECT) {
          key = LCDKEY_NONE;
          status = LCD_SELECT;
        } else status = LCD_KEYOK;
      }
      else key = LCDKEY_NONE;
      break;
    case LCD_NOKEY:
      if (key != LCDKEY_NONE) {
        millis_old = millis();
        key = LCDKEY_NONE;
        status = key == LCDKEY_SELECT ? LCD_SELECT : LCD_KEY;
      } else {
        if (((millis() - millis_old) & 0x7FFFFFFF) > EXIT) {
          key = LCDKEY_EXIT;
          status = LCD_EXIT;
        }
      }
      break;
    case LCD_KEYOK:
      if (key == LCDKEY_NONE) {
        millis_old = millis();
        status = LCD_NOKEY;
      }
      else {
        if (((millis() - millis_old) & 0x7FFF) > DELAY) {
          millis_old = millis();
          status = LCD_REPEAT;
        }
        else key = LCDKEY_NONE;
      }
      break;
    case LCD_REPEAT:
      if (key == LCDKEY_NONE) {
        millis_old = millis();
        status = LCD_NOKEY;
      } else {
        if (((millis() - millis_old) & 0x7FFF) > REPEAT) millis_old = millis();
        else key = LCDKEY_NONE;
      }
      break;
    case LCD_SELECT:
      if (key == LCDKEY_NONE) {
        key = ((millis() - millis_old) & 0x7FFF) > DELAY ? LCDKEY_ESC : LCDKEY_SELECT;
        millis_old = millis();
        status = LCD_NOKEY;
      } else key = LCDKEY_NONE;
      break;
  }
  return key;
}
